#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <malloc.h>
#include <string.h>
#include <errno.h>

static struct sig_str_to_num {
	const char	*name;
	int		num;
} signals[] = {
	{ "HUP",	SIGHUP },
	{ "INT",	SIGINT },
	{ "QUIT",	SIGQUIT },
	{ "TRAP",	SIGTRAP },
	{ "ABRT",	SIGABRT },
	{ "BUS",	SIGBUS },
	{ "FPE",	SIGFPE },
	{ "KILL",	SIGKILL },
	{ "USR1",	SIGUSR1 },
	{ "SEGV",	SIGSEGV },
	{ "USR2",	SIGUSR2 },
	{ "PIPE",	SIGPIPE },
	{ "ALRM",	SIGALRM },
	{ "TERM",	SIGTERM },
	{ "RTMIN",	SIGSYS+3 },
	{ NULL, -1 }
};

static int sig_name_to_num(const char *name)
{
	unsigned int i;

	for (i = 0; signals[i].name != NULL; i++) {
		if (!strncasecmp(signals[i].name, name, strlen(signals[i].name)))
			return signals[i].num;
	}

	return -1;
}

static void print_usage()
{
	printf("Usage: my_kill [options] pid ...\n"
	" -s		signal name\n"
	" -d		data to send\n");
}

#define OPT_SIG_NUM	0x01
#define OPT_SIG_DATA	0x02

int main(int argc, char **argv)
{
	union sigval value;
	int opt, sig_num, data, res, options;
	pid_t pid;
	unsigned int i;

	options = 0;
	while ((opt = getopt(argc, argv, "hs:d:")) != -1) {
		switch (opt) {
		case 'h':
			print_usage();
			exit(0);
			break;
		case 's':
			sig_num = sig_name_to_num(optarg);
			options |= OPT_SIG_NUM;
			break;
		case 'd':
			data = atoi(optarg);
			options |= OPT_SIG_DATA;
			break;
		default:
			print_usage();
			exit(1);
			break;
		}
	}

	if (!(options & OPT_SIG_DATA) || !(options & OPT_SIG_NUM)) {
		print_usage();
		exit(0);
	}

	if (sig_num == -1) {
		fprintf(stderr, "wrong signal number");
		exit(1);
	}

	if (argc-optind == 0) {
		fprintf(stderr, "wait process pid");
		exit(1);
	}

	value.sival_int = data;
	for (i = optind; i < argc; i++) {
		pid = atoi(argv[i]);
		printf("pid -- %d, sig -- %d, value -- %d\n", pid, sig_num, value.sival_int);
		res = sigqueue(pid, sig_num, value);
		if (res == -1)
			perror("sigqueue failed");
	}

	return 0;
}

